@@ -1,12 +1,12 @@
 import sys
 
-help_message = '''
+from cameo_claw import lst_demo_url
+
+help_message = f'''
 python3 -m pip install cameo_claw --upgrade
 python3 -c "
 from cameo_claw import it_filter_group
-lst_url=[
-'https://cameo-claw-data.vercel.app/device_21152332479_daily_2022-03-30.csv',
-'https://cameo-claw-data.vercel.app/device_21152332479_daily_2022-03-31.csv']
+lst_url={lst_demo_url}
 target_directory = './data/topic_filter_group/'
 int_total=len(lst_url)
 lst_distinct_column = ['deviceId', 'localTime', 'sensorId']
