@@ -8,7 +8,7 @@ from cameo_claw.file import mkdir
 from cameo_claw.net import url_to_filename, requests_get_ram_cache
 
 import warnings
-import pandas as pd
+# import pandas as pd
 from cameo_claw.functional import it_mp_f
 
 warnings.filterwarnings("ignore")
@@ -65,13 +65,13 @@ def pl_read_csv_str(bytes1):
     return pl.read_csv(bytes1, infer_schema_length=0).with_columns(pl.all().cast(pl.Int32, strict=False))
 
 
-def pd_read_csv(bytes1):
-    df = None
-    if bytes1[0:2] == b'\x1f\x8b':
-        df = pd.read_csv(BytesIO(bytes1), compression='gzip')
-    else:
-        df = pd.read_csv(BytesIO(bytes1))
-    return df
+# def pd_read_csv(bytes1):
+#     df = None
+#     if bytes1[0:2] == b'\x1f\x8b':
+#         df = pd.read_csv(BytesIO(bytes1), compression='gzip')
+#     else:
+#         df = pd.read_csv(BytesIO(bytes1))
+#     return df
 
 
 def groupby_write_gzip(df, url, lst_group_by_column, target_directory, has_header=True):
