@@ -1,3 +1,4 @@
+import httpx
 import requests
 import os
 
@@ -21,3 +22,13 @@ def requests_get_bytes(url):
 
 def requests_get_ram_cache(f, url, target_directory):
     return f(requests_get_bytes(url))
+
+
+async def async_post(url, dic):
+    async with httpx.AsyncClient() as c:
+        return (await c.post(url, data=dic)).json()
+
+
+async def async_get(url):
+    async with httpx.AsyncClient() as c:
+        return (await c.get(url)).json()
